@@ -542,177 +542,6 @@ class DBCommon(object):
 
         return et
 
-def bufferkeypipa(command, data):
-    """ Do not save password to the buffer! """
-    command = command + " v5" #add version
-    if data != None:
-        data = data.copy()
-        if pipaparpass in data:
-            data.pop(pipaparpass)
-        return command + " " +  urllib.urlencode(sorted(data.items()))
-    else:
-        return command
-
-class PIPA(DBCommon):
-
-    def __init__(self, address=defaddresspipa, cache=None, username=None, password=None):
-        self.address = address
-        self.db=DBInterface(address)
-        self.db_coverage=DBInterface(defaddresspipa_coverage) #temporary
-        self.buffer = cache
-        self.username = None
-        if username != None:
-            self.username = username
-            self.password = password
-
-    def add_auth(self, data=None):
-        if self.username == None:
-            return data
-        authdic = { pipaparuser: self.username, pipaparpass: self.password }
-        if data != None:
-            authdic.update(data)
-        return authdic
-
-    def annotations(self, reload=False, bufver="0"):
-        """
-        Returns a dictionary of (id: dictionary of annotations).
-        """
-        #res, legend = self.sq("mapping_list", data=self.add_auth(), reload=reload, bufferkey=bufferkeypipa, bufver=bufver)
-        res, legend = self.sq("", data=self.add_auth({"action": "expression_list"}), reload=reload, bufferkey=bufferkeypipa, bufver=bufver)
-        return dict( (sa[0], dict(zip(legend[1:], sa[1:]))) for sa in res )
-
-    def chips(self, ids, ctype, reload=False, bufver="0"):
-        def separatefn(res):
-            #each one is own rown
-            #genes are in the first row
-            #remove unknown
-            genes = res[0][1:]
-            cids = nth(res,0)[1:]
-
-            antss = {}
-            for i,cid in enumerate(cids):
-                row = i+1
-                vals = res[row][1:]
-                antss[cid] = [ list(a) for a in zip(genes, vals) if a[1] != "?" ]
-            return ['gene_id', 'value'], antss
-
-        download_command = "gene_expression"
-        datadict = {"action": download_command, "ids":"$MULTI$", 'transpose':'1'}
-        if ctype != None:
-            datadict["type"] = ctype
-        antss = self.downloadMulti("", ids, data=self.add_auth(datadict), chunk=10, separatefn=separatefn, bufferkey=bufferkeypipa, bufreload=reload, bufver=bufver)
-        for a,legend in antss:
-            yield a
-
-    def coverage(self, id_, genome, chromosome, reload=False, bufver="0", raw=False):
-        data = self.add_auth({"action": "coverage_strand", "chr_name": str(chromosome), 
-            "pixel_size": "1", "genome": genome, "sample_id": str(id_), "raw": str(raw).lower()})
-
-        def comp():
-
-            d = dict(data)
-
-            def get():
-                #sq returns [ "all lines after first", "firstline"] - take first column!
-                #do not use buffering on this stage!
-                a = self.sq("", data=d, reload=reload, buffer=False, bufferkey=bufferkeypipa, bufver=bufver, db=self.db_coverage)[1][0].strip("[]")
-                return map(int, a.split(", "))
-                
-            d["strand"] = "plus"
-            plus = get()
-            d["strand"] = "minus"
-            minus = get()
-
-            import numpy
-            return numpy.array([plus, minus])
-        
-        bufkey =  bufferkeypipa("", data)
-        return self.bufferFun(bufkey, bufver, reload, comp)
-
-
-    def gene_expression_types(self, reload=False, bufver="0"):
-        #res, legend = self.sq("gene_expression_type", data={self.add_auth(), reload=reload, bufferkey=bufferkeypipa, bufver=bufver)
-        res, legend = self.sq("", data=self.add_auth({"action":"gene_expression_type"}), reload=reload, bufferkey=bufferkeypipa, bufver=bufver)
-        return sorted(tuple(a) for a in res)
-
-    def chips_keynaming(self, ctype):
-        keynamingfn,_ = self.downloadMulti_bufcommand_replace_multi("", data=self.add_auth({"action": "gene_expression", "ids":"$MULTI$", 'transpose':'1', 'type':ctype}), chunk=100, bufferkey=bufferkeypipa, transformfn=None)
-        return keynamingfn
-
-    def get_data(self, exclude_constant_labels=False, average=median, 
-        ids=None, callback=None, bufver="0", transform=None, ctype=None, allowed_labels=None):
-        """
-        Get data in a single example table with labels of individual attributes
-        set to annotations for query and post-processing
-        instructions.
-
-        Parameters: 
-            average: function used for combining multiple reading of the same spot on
-                a chip. If None, no averaging is done. Fuction should take a list
-                of floats and return an "averaged" float.
-            ids: a list of chip ids. If absent, make a search
-            exclude_constant_labels: if a label has the same value in whole 
-                example table, remove it
-            format: if short, use short format for chip download
-            ctype: expression type, from gene_expression_types
-
-        Defaults: Median averaging.
-        """
-
-        def optcb():
-            if callback: callback()
-
-        cbc = CallBack(1, optcb, callbacks=10)
-
-        if not ids:
-            return None
-
-        cbc.end()
-
-        #downloads annotations
-        cbc = CallBack(len(ids), optcb, callbacks=10)
-
-        readall = self.annotations()
-
-        read = {}
-        for a,b in readall.items():
-            read[a] = b.items()
-
-        cbc.end()
-
-        #till now downloads were small
-
-        import time
-        tstart = time.time()
-
-        #here download actually happens
-        chipfn = None
-
-        chipfn = lambda x: self.chips(x, ctype, bufver=bufver)
-       
-        if verbose:
-            print "DOWNLOAD TIME", time.time() - tstart
-
-        cbc = CallBack(len(ids)*2+len(ids)+1, optcb, callbacks=999-30)
-        et = self.exampleTables(ids, spotmap={}, callback=cbc, annots=read, exclude_constant_labels=exclude_constant_labels, chipfn=chipfn, allowed_labels=allowed_labels)
-        cbc.end()
-
-        cbc = CallBack(1, optcb, callbacks=10)
-
-        #transformation is performed prior to averaging
-        if transform != None:
-            transformValues(et, fn=transform) #in place transform
-            cbc()
-
-        #if average function is given, use it to join same spotids
-        if average != None:
-            et = averageAttributes(et, fn=average)
-            cbc()
-
-        cbc.end()
-
-        return et
-
 def bufferkeypipax(command, data):
     """ Do not save password to the buffer! """
     command = command + " v8" #add version
@@ -725,7 +554,7 @@ def bufferkeypipax(command, data):
         return command
 
 
-class PIPAx(PIPA):
+class PIPAx(DBCommon):
     """ An interface to `PIPAx API <http://pipa.biolab.si/hp/api.html>`_.
     """
 
@@ -734,9 +563,11 @@ class PIPAx(PIPA):
     def __init__(self, address=API_ADDRESS, cache=None,
                  username=None, password=None):
         """
-        :param address: The address of the API. 
-        :param username:
-        :param password: Your login details, leave None for default settings.
+        :param str address: The address of the API. 
+        :param str username:
+        :param str password: Login info; None for public access.
+        :param CacheSQLite cache: A cache that stores results locally (an
+            :obj:`CacheSQLite`).
         """
         self.address = address
         self.db = DBInterface(address)
@@ -744,6 +575,14 @@ class PIPAx(PIPA):
         self.username = username
         self.password = password
 
+    def add_auth(self, data=None):
+        if self.username == None:
+            return data
+        authdic = { pipaparuser: self.username, pipaparpass: self.password }
+        if data != None:
+            authdic.update(data)
+        return authdic
+
     def genomes(self, reload=False, bufver="0"):
         """Return a list of available genomes as a list of
         (genome_id, genome_name) tuples.
@@ -753,28 +592,30 @@ class PIPAx(PIPA):
         return [tuple(r) for r in res]
 
     def mappings(self, reload=False, bufver="0"):
-        """Return a dictionary of {unique_id: dictionary_of_annotations}
+        """Return available mappings as dictionary of 
+        { mapping_id: dictionary_of_annotations }
         where the keys for dictionary_of_annotations are
-        ["id", data_id", "data_name", "genomes_id"]
+        "id", data_id", "data_name", "genomes_id".
         """
         data = self.add_auth({"action": "mappings"})
         res, legend = self.sq("", data=data, reload=reload,
                               bufferkey=bufferkeypipax,
                               bufver=bufver)
-
         return dict((sa[0], dict(zip(legend[1:], sa[1:]))) for sa in res)
 
     def result_types(self, reload=False, bufver="0"):
-        """Return a list of available result type templates.
+        """Return a list of available result types.
         """
         data = {"action": "results_templates"}
         res, _ = self.sq("", data=data, reload=reload, bufver=bufver)
         return sorted(tuple(a) for a in res)
 
     def results_list(self, rtype, reload=False, bufver="0"):
-        """Return a list of available results for a result type template
-        `rtype` (a key from the `result_types` return value).
+        """Return a list of available gene expressions for a specific
+        result type. Returns a dictionary, where the keys are ID
+        and values are dictionaries of sample annotations.
 
+        :param str rtype: Result type to use (see :obj:`result_types`).
         """
         data = {"action": "results",
                 "results_templates_id": rtype}
@@ -801,29 +642,28 @@ class PIPAx(PIPA):
                  callback=None, bufver="0", transform=None,
                  allowed_labels=None, reload=False):
         """
-        Get data in a single example table with labels of individual
-        attributes set to annotations for query and post-processing
-        instructions.
+        Return data in a :obj:`Orange.data.Table`. Each feature represents 
+        a sample and each row is a gene. The feature's ``.attributes`` 
+        contain annotations.
 
-        :param ids: If `result_type` is specified then this must
-            be a list of  `unique_id`s or (id, data_id) tuples as
-            returned by `mappings`. Else the `ids` must be a list
-            of `unique_id`s as returned by `results_list`.
-        :type ids: list
+        :param list ids: List of ids as returned by :obj:`results_list`
+            if `result_type` is None; list of ids as returned by :obj:`mappings` 
+            if `result_type` is set.
 
-        :param result_type: Result template type id as returned by
-             `result_types`. Not specified by default (see `ids`.)
-        :type result_type: str
+        :param str result_type: Result template type id as returned by
+             :obj:`result_types`.
 
-        :param exclude_constant_labels: If a label has the same value
+        :param bool exclude_constant_labels: If a label has the same value
             in whole example table, remove it.
-        :type exclude_constant_labels: bool
 
-        :param average: Function used for combining multiple reading of
-            the same spot on a chip. If None, no averaging is done.
+        :param function average: Function that combines multiple reading of
+            the same gene on a chip. If None, no averaging is done.
             Function should take a list of floats and return an "averaged"
-            float (default `median`).
-        :type average: function
+            float (the default functions returns the median).
+
+        :param function transform: A function that transforms individual values.
+            It should take and return a float. Example use: logarithmic 
+            transformation. Default: None.
 
         """
 
@@ -1003,7 +843,8 @@ Can only retrieve a single result_template_type at a time"""
 
 class DictyExpress(DBCommon):
     """
-    Type is object id
+    Access the `DictyExpress data API 
+    <http://bcm.fri.uni-lj.si/microarray/api/index.php>`_.
     """
     
     aoidPairs = txt2ll("""time extractions.developmental_time_point
@@ -1024,6 +865,12 @@ extractions extractions
 chips chips""")
 
     def __init__(self, address=defaddress, cache=None):
+        """
+        :param str address: The address of the API. 
+        :param cache: A cache that stores results locally (an instance
+            of :obj:`CacheSQLite`).
+        """
+        
         self.address = address
         self.db = DBInterface(address)
         self.buffer = cache
@@ -1070,10 +917,10 @@ chips chips""")
 
     def annotationOptions(self, ao=None, onlyDiff=False, **kwargs):
         """
-        Returns annotation options for given query. Returns all possible 
+        Return annotation options for given query. Return all possible 
         annotations if the query is omitted.
 
-        If ao is choosen, only result
+        If `ao` is chosen, only return options for that object id.
         """
         params = ""
         if len(kwargs) > 0: params += "&query=" + self.pq(kwargs)
@@ -1119,7 +966,7 @@ chips chips""")
         """
         Return annotations for specified type and ids.
 
-        :param type: Object type (:obj:`DictyExpress.objects`). 
+        :param type: Object type (see :obj:`objects`). 
         :param ids: If set, only annotations corresponding to the given
           ids are returned. Annotations are in the same order as
           input ids.
@@ -1155,23 +1002,23 @@ chips chips""")
 
     def search(self, type, **kwargs):
         """
-        Break search for multiple values of one attribute to independant searches.
-        Search is case insensitive.
+        Search the database. Search is case insensitive.
         
-        List of searchable annotation types: self.saoids.keys()
+        :param type: Annotation type (list them
+                with ``DictyExpress().saoids.keys()``).
+        :param kwargs: In the form ``annotation=values``. Values
+            can are either strings or a list of strings (interpreted as
+            an OR operator between list elements).
 
-        example usage:
+        The following example lists ids of normalized entries where platform 
+        is minichip and sample is abcC3-::
 
-        search("norms", platform='minichip', sample='abcC3-')
+            search("norms", platform='minichip', sample='abcC3-')
 
-        finds all ids of normalized entries where platform is minchip and
-        sample is abcC3-
+        The following example lists ids of normalized entries where platform 
+        is minichip and sample is abcC3- or abcG15-::
 
-        search("norms", platform='minichip', sample=[ 'abcC3-', 'abcG15-'])
-
-        finds all ids of normalized entries where platform is minichip and
-        sample is abcC3- or those where platform is minichip and sample
-        is abcG15-
+            search("norms", platform='minichip', sample=[ 'abcC3-', 'abcG15-'])
         """
         
         #search for all combinations of values - this is slow!
@@ -1205,7 +1052,7 @@ chips chips""")
   
     def chipNs(self, ids, remove_bad=True):
         """
-        If removebad = True removes those with weights 0.
+        If removebad == True removes those with weights 0.
         """
           
         def sel(res, legend):
@@ -1297,7 +1144,7 @@ chips chips""")
 
     def __annotationTypes(self):
         """
-        Returns list of [ annotation_object_id, name ]
+        Return a list of [ annotation_object_id, name ].
         """
         res, legend = self.sq("action=get_annotation_types")
         res2 = onlyColumns(res, legend, ["annotation_object_id", 'name'])
@@ -1328,21 +1175,27 @@ chips chips""")
     def get_data(self, type="norms", exclude_constant_labels=False, average=median, 
         ids=None, callback=None, format="short", transform=None, allowed_labels=None, **kwargs):
         """
-        Return data in a single table. Each feature represents a chip
-        and each sample is a gene. The ``.attributes`` dictionaries of
-        the features contain annotations.
+        Return data in a :obj:`Orange.data.Table`. Each feature is a sample
+        and each row(:obj:`Orange.data.Instance` is a gene. 
+        The feature's ``.attributes`` contain annotations.
+
+        :param list ids: A list of chip ids. If absent, make a search. In this case
+          any additional keyword arguments are threated as in :obj:`search`.
 
-        :param average: A function that combines multiple reading of the same spot on
-          a chip. If None, no averaging is done. Fuction should take a list
-          of floats and return an "averaged" float. Default: median.
-        :param ids: A list of chip ids. If absent, make a search. In this case
-          any additional parameters are threated as in :obj:`DictyExpress.search`.
         :param exclude_constant_labels: Remove labels if they have the  same value 
           for the whole table. 
+        
         :param str format: If "short", use short format for downloads.
-        :param transform: A function that transforms individual values: it should take
-          a float and return a float. An example use would be a logarithmic 
-          transformation. Default: None.
+
+        :param function average: Function that combines multiple reading of
+            the same gene on a chip. If None, no averaging is done.
+            Function should take a list of floats and return an "averaged"
+            float (the default functions returns the median).
+
+        :param function transform: A function that transforms individual values.
+            It should take and return a float. Example use: logarithmic 
+            transformation. Default: None.
+
 
         :returns: Chips with given ids in a single data table.
         :rtype: :obj:`Orange.data.Table`
@@ -1867,7 +1720,6 @@ if __name__=="__main__":
         et.save("ett.tab")
         print open("ett.tab").read()
 
-    """
     a = DictyBase()
     print len(a.info)
 
@@ -1881,31 +1733,7 @@ if __name__=="__main__":
         count += 1
         #print "CBBB", count
 
-    et = dbc.get_single_data(sample=[ "tagA-", "pkaC-"], callback=cb, exclude_constant_labels=True, allowed_labels=["sample"])
+    et = dbc.get_data(sample=[ "tagA-", "pkaC-"], callback=cb, exclude_constant_labels=True, allowed_labels=["sample"])
     print et.domain
     print et.domain[0].attributes
     printet(et)
-    """
-
-    d = PIPA(cache=CacheSQLite("../tmpbufnewpipa"))
-
-    print d.gene_expression_types()
-
-    cov = d.coverage("777", chromosome=1, genome="dd")
-    print cov
-
-    allids = d.annotations().keys()
-    allids = allids[:1]
-    print ("list", allids)
-    print d.annotations().items()[0]
-    print ("annots", d.annotations().items()[:2])
-
-    allids = map(str, allids)
-
-    import math
-
-    data = d.get_data(ids=allids, ctype="3")
-
-    #data = d.get_data(ids=allids, transform=lambda x: math.log(x+1, 2), allowed_labels=["strain"], ctype="3")
-
-    printet(data)
